@@ -22,13 +22,14 @@
 #include <strings.h>
 #endif
 
+#include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/serialize.h>
 #include <net-snmp/agent/read_only.h>
 
 typedef struct netsnmp_num_file_instance_s {
     char *file_name;
     FILE *filep;
-    int   type;
+    u_char type;
     int   flags;
 } netsnmp_num_file_instance;
 
@@ -140,6 +141,7 @@ get_reg(const char *name,
     return myreg;
 }
 
+/* Watched 'long' instances are writable on both 32-bit and 64-bit systems  */
 int
 netsnmp_register_read_only_ulong_instance(const char *name,
                                           const oid * reg_oid,
@@ -211,7 +213,7 @@ netsnmp_register_long_instance(const char *name,
                    (void *)it, sizeof(long), ASN_INTEGER, WATCHER_FIXED_SIZE));
 }
 
-
+/* Watched 'int' instances are only writable on 32-bit systems  */
 int
 netsnmp_register_read_only_uint_instance(const char *name,
                                          const oid * reg_oid,
@@ -482,6 +484,15 @@ netsnmp_register_int_instance(const char *name,
                    (void *)it, sizeof(int), ASN_INTEGER, WATCHER_FIXED_SIZE));
 }
 
+#ifdef HAVE_DMALLOC_H
+static void free_wrapper(void * p)
+{
+    free(p);
+}
+#else
+#define free_wrapper free
+#endif
+
 #ifndef NETSNMP_NO_DEPRECATED_FUNCTIONS
 
 /**
@@ -531,7 +542,7 @@ netsnmp_instance_ulong_handler(netsnmp_mib_handler *handler,
         netsnmp_request_add_list_data(requests,
                                       netsnmp_create_data_list
                                       (INSTANCE_HANDLER_NAME, it_save,
-                                       free));
+                                       &free_wrapper));
         break;
 
     case MODE_SET_ACTION:
@@ -650,7 +661,7 @@ netsnmp_instance_long_handler(netsnmp_mib_handler *handler,
         netsnmp_request_add_list_data(requests,
                                       netsnmp_create_data_list
                                       (INSTANCE_HANDLER_NAME, it_save,
-                                       free));
+                                       &free_wrapper));
         break;
 
     case MODE_SET_ACTION:
@@ -733,7 +744,7 @@ netsnmp_instance_int_handler(netsnmp_mib_handler *handler,
         netsnmp_request_add_list_data(requests,
                                       netsnmp_create_data_list
                                       (INSTANCE_HANDLER_NAME, it_save,
-                                       free));
+                                       &free_wrapper));
         break;
 
     case MODE_SET_ACTION:
@@ -849,7 +860,7 @@ netsnmp_instance_num_file_handler(netsnmp_mib_handler *handler,
         netsnmp_request_add_list_data(requests,
                                       netsnmp_create_data_list
                                       (INSTANCE_HANDLER_NAME, it_save,
-                                       free));
+                                       &free_wrapper));
         break;
 
     case MODE_SET_ACTION:
@@ -949,7 +960,7 @@ netsnmp_instance_uint_handler(netsnmp_mib_handler *handler,
         netsnmp_request_add_list_data(requests,
                                       netsnmp_create_data_list
                                       (INSTANCE_HANDLER_NAME, it_save,
-                                       free));
+                                       &free_wrapper));
         break;
 
     case MODE_SET_ACTION:
