@@ -130,7 +130,7 @@ int             header_hrswinst(struct variable *, oid *, size_t *, int,
 int             header_hrswInstEntry(struct variable *, oid *, size_t *,
                                      int, size_t *, WriteMethod **);
 
-extern struct timeval starttime;
+#define starttime (*(const struct timeval*)netsnmp_get_agent_starttime())
 
         /*********************
 	 *
@@ -140,7 +140,7 @@ extern struct timeval starttime;
 extern void     Init_HR_SWInst(void);
 extern int      Get_Next_HR_SWInst(void);
 extern void     End_HR_SWInst(void);
-extern void     Save_HR_SW_info(int ix);
+extern int      Save_HR_SW_info(int ix);
 
 #ifdef HAVE_LIBRPM
 static void     Mark_HRSW_token(void);
@@ -299,6 +299,7 @@ header_hrswInstEntry(struct variable *vp,
     oid             newname[MAX_OID_LEN];
     int             swinst_idx, LowIndex = -1;
     int             result;
+    int             err = 0, errcount = 0;
 
     DEBUGMSGTL(("host/hr_swinst", "var_hrswinstEntry: "));
     DEBUGMSGOID(("host/hr_swinst", name, *length));
@@ -309,29 +310,38 @@ header_hrswInstEntry(struct variable *vp,
      * Find "next" installed software entry 
      */
 
-    Init_HR_SWInst();
-    while ((swinst_idx = Get_Next_HR_SWInst()) != -1) {
-        DEBUGMSG(("host/hr_swinst", "(index %d ....", swinst_idx));
+    do {
+        Init_HR_SWInst();
+        while ((swinst_idx = Get_Next_HR_SWInst()) != -1) {
+            DEBUGMSG(("host/hr_swinst", "(index %d ....", swinst_idx));
 
-        newname[HRSWINST_ENTRY_NAME_LENGTH] = swinst_idx;
-        DEBUGMSGOID(("host/hr_swinst", newname, *length));
-        DEBUGMSG(("host/hr_swinst", "\n"));
-        result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
-        if (exact && (result == 0)) {
-            LowIndex = swinst_idx;
-            Save_HR_SW_info(LowIndex);
-            break;
-        }
-        if ((!exact && (result < 0)) &&
-            (LowIndex == -1 || swinst_idx < LowIndex)) {
-            LowIndex = swinst_idx;
-            Save_HR_SW_info(LowIndex);
+            newname[HRSWINST_ENTRY_NAME_LENGTH] = swinst_idx;
+            DEBUGMSGOID(("host/hr_swinst", newname, *length));
+            DEBUGMSG(("host/hr_swinst", "\n"));
+            result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
+            if (exact && (result == 0)) {
+                LowIndex = swinst_idx;
+                err = Save_HR_SW_info(LowIndex);
+                break;
+            }
+            if ((!exact && (result < 0)) &&
+                (LowIndex == -1 || swinst_idx < LowIndex)) {
+                LowIndex = swinst_idx;
+                err = Save_HR_SW_info(LowIndex);
 #ifdef HRSWINST_MONOTONICALLY_INCREASING
-            break;
+                break;
 #endif
+            }
         }
+        if (err != 0 )
+            errcount++;
+    /* restart until Save_HR_SW_info() succeeds,max. 3 times */
+    } while (err != 0 && errcount < 3); 
+    if (err != 0) {
+        DEBUGMSGTL(("host/hr_swinst", "restart did not help, bailing out\n"));
+        return (MATCH_FAILED);
     }
-
+	    
     Mark_HRSW_token();
     End_HR_SWInst();
 
@@ -428,7 +438,8 @@ var_hrswinst(struct variable * vp,
 #ifdef HAVE_PKGINFO
             pver = pkgparam(swi->swi_name, "VERSION");
             /* 1 spot for the terminating null and one for the dash */
-            if (strlen(pver) + 2 + strlen(string) <= sizeof(string)) {
+            if (pver && 
+               (strlen(pver) + 2 + strlen(string) <= sizeof(string))) {
                 strcat(string, "-");
                 strcat(string, pver);
             }
@@ -646,7 +657,7 @@ Get_Next_HR_SWInst(void)
     return -1;
 }
 
-void
+int
 Save_HR_SW_info(int ix)
 {
     SWI_t          *swi = &_myswi;      /* XXX static for now */
@@ -675,8 +686,13 @@ Save_HR_SW_info(int ix)
         h = rpmdbGetRecord(swi->swi_rpmdb, offset);
 #endif
 
-        if (h == NULL)
-            return;
+        if (h == NULL) {
+            DEBUGMSGTL(("host/hr_swinst",
+                    "RPM cache has probably expired when reading entry %d, "
+                    "reloading...\n", ix));
+            swi->swi_timestamp = 0;
+            return -1;
+        }
         if (swi->swi_h != NULL)
             headerFree(swi->swi_h);
         swi->swi_h = h;
@@ -694,6 +710,7 @@ Save_HR_SW_info(int ix)
     snprintf(swi->swi_name, sizeof(swi->swi_name), "%s", swi->swi_dep->d_name);
     swi->swi_name[ sizeof(swi->swi_name)-1 ] = 0;
 #endif
+    return 0;
 }
 
 #ifdef	HAVE_LIBRPM
