@@ -1,7 +1,7 @@
 /*
  *  Interface MIB architecture support
  *
- * $Id: ipaddress_linux.c 17255 2008-10-14 09:44:26Z jsafranek $
+ * $Id: ipaddress_linux.c 19548 2010-10-25 08:06:37Z bvassche $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -21,20 +21,18 @@
 #if defined (NETSNMP_ENABLE_IPV6)
 #include <linux/types.h>
 #include <asm/types.h>
-#if defined(HAVE_PTHREAD_H) && defined(HAVE_LINUX_RTNETLINK_H)
+#if defined(HAVE_LINUX_RTNETLINK_H)
 #include <linux/netlink.h>
-#include <pthread.h>
 #include <linux/rtnetlink.h>
 #ifdef RTMGRP_IPV6_PREFIX
 #define SUPPORT_PREFIX_FLAGS 1
 #endif /* RTMGRP_IPV6_PREFIX */
-#endif /* HAVE_PTHREAD_H && HAVE_LINUX_RTNETLINK_H */
+#endif /* HAVE_LINUX_RTNETLINK_H */
 #endif
 
 #include "ipaddress_ioctl.h"
 #ifdef SUPPORT_PREFIX_FLAGS
 extern prefix_cbx *prefix_head_list;
-extern pthread_mutex_t prefix_mutex_lock;
 #endif
 int _load_v6(netsnmp_container *container, int idx_offset);
 #ifdef HAVE_LINUX_RTNETLINK_H
@@ -247,7 +245,7 @@ _load_v6(netsnmp_container *container, int idx_offset)
          * F: flags (see include/linux/rtnetlink.h, net/ipv6/addrconf.c)
          * I: interface
          */
-        rc = sscanf(line, "%39s %04x %02x %02x %02x %" SNMP_MACRO_VAL_TO_STR(IFNAMSIZ) "s\n",
+        rc = sscanf(line, "%39s %08x %08x %04x %02x %" SNMP_MACRO_VAL_TO_STR(IFNAMSIZ) "s\n",
                     addr, &if_index, &pfx_len, &scope, &flags, if_name);
         if( 6 != rc ) {
             snmp_log(LOG_ERR, PROCFILE " data format error (%d!=6), line ==|%s|\n",
@@ -374,7 +372,7 @@ _load_v6(netsnmp_container *container, int idx_offset)
 #endif
 #ifdef SUPPORT_PREFIX_FLAGS
         memset(&prefix_val, 0, sizeof(prefix_cbx));
-        if(net_snmp_find_prefix_info(&prefix_head_list, addr, &prefix_val, &prefix_mutex_lock) < 0) {
+        if(net_snmp_find_prefix_info(&prefix_head_list, addr, &prefix_val) < 0) {
            DEBUGMSGTL(("access:ipaddress:container", "unable to find info\n"));
            entry->ia_onlink_flag = 1;  /*Set by default as true*/
            entry->ia_autonomous_flag = 2; /*Set by default as false*/
@@ -478,11 +476,11 @@ netsnmp_access_other_info_get(int index, int family)
         if(index == rtmp->ifa_index){
            for (; RTA_OK(rtatp, rtattrlen); rtatp = RTA_NEXT(rtatp, rtattrlen)) {
                 if(rtatp->rta_type == IFA_BROADCAST){
-                   addr.inp = (struct in_addr *)RTA_DATA(rtatp);
+                   addr.addr = ((struct in_addr *)RTA_DATA(rtatp))->s_addr;
                    addr.bcastflg = 1;
                 }
                 if(rtatp->rta_type == IFA_ANYCAST){
-                   addr.inp = (struct in_addr *)RTA_DATA(rtatp);
+                   addr.addr = ((struct in_addr *)RTA_DATA(rtatp))->s_addr;
                    addr.anycastflg = 1;
                 }
            }
