@@ -1,7 +1,7 @@
 /*
  *  ipSystemStatsTable and ipIfStatsTable interface MIB architecture support
  *
- * $Id: systemstats_linux.c 17689 2009-07-16 12:38:15Z jsafranek $
+ * $Id: systemstats_linux.c 19699 2010-11-30 04:58:34Z hardaker $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -116,7 +116,7 @@ _systemstats_v4(netsnmp_container* container, u_int load_flags)
     if (!(devin = fopen("/proc/net/snmp", "r"))) {
         DEBUGMSGTL(("access:systemstats",
                     "Failed to load Systemstats Table (linux1)\n"));
-        snmp_log(LOG_ERR, "cannot open /proc/net/snmp ...\n");
+        NETSNMP_LOGONCE((LOG_ERR, "cannot open /proc/net/snmp ...\n"));
         return -2;
     }
 
@@ -160,7 +160,8 @@ _systemstats_v4(netsnmp_container* container, u_int load_flags)
         while (*stats == ' ') /* skip spaces before stats */
             stats++;
 
-        entry = netsnmp_access_systemstats_entry_create(1, 0);
+        entry = netsnmp_access_systemstats_entry_create(1, 0,
+                    "ipSystemStatsTable.ipv4");
         if(NULL == entry) {
             netsnmp_access_systemstats_container_free(container,
                                                       NETSNMP_ACCESS_SYSTEMSTATS_FREE_NOFLAGS);
@@ -253,8 +254,13 @@ _systemstats_v4(netsnmp_container* container, u_int load_flags)
         /*
          * add to container
          */
-        CONTAINER_INSERT(container, entry);
+        if (CONTAINER_INSERT(container, entry) < 0)
+        {
+            DEBUGMSGTL(("access:systemstats:container","error with systemstats_entry: insert into container failed.\n"));
+            netsnmp_access_systemstats_entry_free(entry);
+        }
     }
+
     return 0;
 }
     
@@ -275,6 +281,7 @@ _additional_systemstats_v4(netsnmp_systemstats_entry* entry,
     if (!(devin = fopen("/proc/net/netstat", "r"))) {
         DEBUGMSGTL(("access:systemstats",
                     "cannot open /proc/net/netstat\n"));
+        NETSNMP_LOGONCE((LOG_ERR,"cannot open /proc/net/netstat\n"));
         return -2;
     }
 
@@ -530,10 +537,10 @@ _systemstats_v6_load_systemstats(netsnmp_container* container, u_int load_flags)
     FILE *devin;
     netsnmp_systemstats_entry *entry = NULL;
     const char     *filename = "/proc/net/snmp6";
-    static int      warned_open = 0;
     int rc = 0;
     
-    entry = netsnmp_access_systemstats_entry_create(2, 0);
+    entry = netsnmp_access_systemstats_entry_create(2, 0,
+            "ipSystemStatsTable.ipv6");
     if(NULL == entry)
         return -3;
     
@@ -544,10 +551,7 @@ _systemstats_v6_load_systemstats(netsnmp_container* container, u_int load_flags)
     if (!(devin = fopen(filename, "r"))) {
         DEBUGMSGTL(("access:systemstats",
                     "Failed to load Systemstats Table (linux1)\n"));
-        if(!warned_open) {
-            ++warned_open;
-            snmp_log(LOG_ERR, "cannot open %s ...\n", filename);
-        }
+        NETSNMP_LOGONCE((LOG_ERR, "cannot open %s ...\n", filename));
         free(entry);
         return 0;
     }
@@ -559,7 +563,12 @@ _systemstats_v6_load_systemstats(netsnmp_container* container, u_int load_flags)
     /*
      * add to container
      */
-    CONTAINER_INSERT(container, entry);
+        if (CONTAINER_INSERT(container, entry) < 0)
+        {
+            DEBUGMSGTL(("access:systemstats:container","error with systemstats_entry: insert into container failed.\n"));
+            netsnmp_access_systemstats_entry_free(entry);
+        }
+
 
     return rc;
 }
@@ -644,7 +653,8 @@ _systemstats_v6_load_ifstats(netsnmp_container* container, u_int load_flags)
             scan_val = strtoull(scan_str, NULL, 0);
         }
         
-        entry = netsnmp_access_systemstats_entry_create(2, scan_val);
+        entry = netsnmp_access_systemstats_entry_create(2, scan_val,
+                "ipIfStatsTable.ipv6");
         if(NULL == entry) {
             fclose(devin);
             closedir(dev_snmp6_dir);
