@@ -33,7 +33,10 @@
 #define IP_STATS_CACHE_TIMEOUT	MIB_STATS_CACHE_TIMEOUT
 #endif
 
-#if defined(HAVE_LIBPERFSTAT_H) && (defined(aix4) || defined(aix5) || defined(aix6)) && !defined(FIRST_PROTOCOL)
+#if defined(HAVE_LIBPERFSTAT_H) && (defined(aix4) || defined(aix5) || defined(aix6) || defined(aix7)) && !defined(FIRST_PROTOCOL)
+#ifdef HAVE_SYS_PROTOSW_H
+#include <sys/protosw.h>
+#endif
 #include <libperfstat.h>
 #ifdef FIRST_PROTOCOL
 perfstat_protocol_t ps_proto;
@@ -225,7 +228,7 @@ init_ip(void)
 #define	USES_SNMP_DESIGNED_IPSTAT
 #endif
 
-#if defined (WIN32) || defined (cygwin)
+#ifdef HAVE_IPHLPAPI_H
 #include <iphlpapi.h>
 #define IP_STAT_STRUCTURE MIB_IPSTATS
 long            ipForwarding;
@@ -270,15 +273,9 @@ ip_handler(netsnmp_mib_handler          *handler,
     /*
      * The cached data should already have been loaded by the
      *    cache handler, higher up the handler chain.
-     * But just to be safe, check this and load it manually if necessary
      */
 #ifdef _USE_PERFSTAT_PROTOCOL
     ip_load(NULL, NULL);
-#elif !defined(hpux11)
-    if (!netsnmp_cache_is_valid(reqinfo, reginfo->handlerName)) {
-        netsnmp_assert(!"cache == valid"); /* always false */
-        ip_load( NULL, NULL );	/* XXX - check for failure */
-    }
 #endif
 
 
@@ -309,7 +306,7 @@ ip_handler(netsnmp_mib_handler          *handler,
         type = ASN_INTEGER;
         break;
     case IPINRECEIVES:
-        ret_value = ipstat.ipInReceives;
+        ret_value = ipstat.ipInReceives & 0xffffffff;
         break;
     case IPINHDRERRORS:
         ret_value = ipstat.ipInHdrErrors;
@@ -327,10 +324,10 @@ ip_handler(netsnmp_mib_handler          *handler,
         ret_value = ipstat.ipInDiscards;
         break;
     case IPINDELIVERS:
-        ret_value = ipstat.ipInDelivers;
+        ret_value = ipstat.ipInDelivers & 0xffffffff;
         break;
     case IPOUTREQUESTS:
-        ret_value = ipstat.ipOutRequests;
+        ret_value = ipstat.ipOutRequests & 0xffffffff;
         break;
     case IPOUTDISCARDS:
         ret_value = ipstat.ipOutDiscards;
@@ -385,7 +382,7 @@ ip_handler(netsnmp_mib_handler          *handler,
         type = ASN_INTEGER;
         break;
     case IPINRECEIVES:
-        ret_value = ipstat.ips_total;
+        ret_value = ipstat.ips_total & 0xffffffff;
         break;
     case IPINHDRERRORS:
         ret_value = ipstat.ips_badsum
@@ -416,7 +413,7 @@ ip_handler(netsnmp_mib_handler          *handler,
 #endif
     case IPINDELIVERS:
 #if HAVE_STRUCT_IPSTAT_IPS_DELIVERED
-        ret_value = ipstat.ips_delivered;
+        ret_value = ipstat.ips_delivered & 0xffffffff;
         break;
 #else
         netsnmp_set_request_error(reqinfo, request, SNMP_NOSUCHOBJECT);
@@ -424,7 +421,7 @@ ip_handler(netsnmp_mib_handler          *handler,
 #endif
     case IPOUTREQUESTS:
 #if HAVE_STRUCT_IPSTAT_IPS_LOCALOUT
-        ret_value = ipstat.ips_localout;
+        ret_value = ipstat.ips_localout & 0xffffffff;
         break;
 #else
         netsnmp_set_request_error(reqinfo, request, SNMP_NOSUCHOBJECT);
