@@ -40,9 +40,6 @@
 #if HAVE_NETDB_H
 #include <netdb.h>
 #endif
-#if HAVE_WINSOCK_H
-#include <winsock.h>
-#endif
 
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
@@ -69,28 +66,31 @@ init_vacm_config_tokens(void) {
                                   "name context model level prefix viewname viewval");
     snmpd_register_config_handler("view", vacm_parse_view, vacm_free_view,
                                   "name type subtree [mask]");
-    snmpd_register_config_handler("vacmView", vacm_parse_config_view, NULL,
-                                  NULL);
-    snmpd_register_config_handler("vacmGroup", vacm_parse_config_group,
-                                  NULL, NULL);
-    snmpd_register_config_handler("vacmAccess", vacm_parse_config_access,
-                                  NULL, NULL);
-    snmpd_register_config_handler("vacmAuthAccess", vacm_parse_config_auth_access,
-                                  NULL, NULL);
+    snmpd_register_const_config_handler("vacmView",
+                                        vacm_parse_config_view, NULL, NULL);
+    snmpd_register_const_config_handler("vacmGroup",
+                                        vacm_parse_config_group,
+                                        NULL, NULL);
+    snmpd_register_const_config_handler("vacmAccess",
+                                        vacm_parse_config_access,
+                                        NULL, NULL);
+    snmpd_register_const_config_handler("vacmAuthAccess",
+                                        vacm_parse_config_auth_access,
+                                        NULL, NULL);
 
     /* easy community auth handler */
     snmpd_register_config_handler("authcommunity",
                                   vacm_parse_authcommunity,
-                                  NULL, "authtype1,authtype2 community [default|hostname|network/bits [oid|-V view]]");
+                                  NULL, "authtype1,authtype2 community [default|hostname|network/bits [oid|-V view [context]]]");
 
     /* easy user auth handler */
     snmpd_register_config_handler("authuser",
                                   vacm_parse_authuser,
-                                  NULL, "authtype1,authtype2 [-s secmodel] user [noauth|auth|priv [oid|-V view]]");
+                                  NULL, "authtype1,authtype2 [-s secmodel] user [noauth|auth|priv [oid|-V view [context]]]");
     /* easy group auth handler */
     snmpd_register_config_handler("authgroup",
                                   vacm_parse_authuser,
-                                  NULL, "authtype1,authtype2 [-s secmodel] group [noauth|auth|priv [oid|-V view]]");
+                                  NULL, "authtype1,authtype2 [-s secmodel] group [noauth|auth|priv [oid|-V view [context]]]");
 
     snmpd_register_config_handler("authaccess", vacm_parse_authaccess,
                                   vacm_free_access,
@@ -115,20 +115,20 @@ void
 init_vacm_snmpd_easy_tokens(void) {
 #if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
     snmpd_register_config_handler("rwcommunity", vacm_parse_rwcommunity, NULL,
-                                  "community [default|hostname|network/bits [oid]]");
+                                  "community [default|hostname|network/bits [oid|-V view [context]]]");
     snmpd_register_config_handler("rocommunity", vacm_parse_rocommunity, NULL,
-                                  "community [default|hostname|network/bits [oid]]");
+                                  "community [default|hostname|network/bits [oid|-V view [context]]]");
 #ifdef NETSNMP_TRANSPORT_UDPIPV6_DOMAIN
     snmpd_register_config_handler("rwcommunity6", vacm_parse_rwcommunity6, NULL,
-                                  "community [default|hostname|network/bits [oid]]");
+                                  "community [default|hostname|network/bits [oid|-V view [context]]]");
     snmpd_register_config_handler("rocommunity6", vacm_parse_rocommunity6, NULL,
-                                  "community [default|hostname|network/bits [oid]]");
+                                  "community [default|hostname|network/bits [oid|-V view [context]]]");
 #endif
 #endif /* support for community based SNMP */
     snmpd_register_config_handler("rwuser", vacm_parse_rwuser, NULL,
-                                  "user [noauth|auth|priv [oid]]");
+                                  "user [noauth|auth|priv [oid|-V view [context]]]");
     snmpd_register_config_handler("rouser", vacm_parse_rouser, NULL,
-                                  "user [noauth|auth|priv [oid]]");
+                                  "user [noauth|auth|priv [oid|-V view [context]]]");
 }
 
 void
@@ -629,7 +629,7 @@ vacm_parse_view(const char *token, char *param)
     size_t          suboid_len = 0;
     size_t          mask_len = 0;
     u_char          viewMask[VACMSTRINGLEN];
-    int             i;
+    size_t          i;
     char            *st;
 
     name = strtok_r(param, " \t\n", &st);
@@ -666,7 +666,7 @@ vacm_parse_view(const char *token, char *param)
         return;
     }
     if (mask) {
-        int             val;
+        unsigned int val;
         i = 0;
         for (mask = strtok_r(mask, " .:", &st); mask; mask = strtok_r(NULL, " .:", &st)) {
             if (i >= sizeof(viewMask)) {
@@ -710,12 +710,13 @@ vacm_gen_com2sec(int commcount, const char *community, const char *addressname,
                  const char *publishtoken,
                  void (*parser)(const char *, char *),
                  char *secname, size_t secname_len,
-                 char *viewname, size_t viewname_len, int version)
+                 char *viewname, size_t viewname_len, int version,
+                 const char *context)
 {
     char            line[SPRINT_MAX_LEN];
 
     /*
-     * com2sec6|comsec anonymousSecNameNUM    ADDRESS  COMMUNITY 
+     * com2sec6|comsec [-Cn CONTEXT] anonymousSecNameNUM    ADDRESS  COMMUNITY 
      */
     snprintf(secname, secname_len-1, "comm%d", commcount);
     secname[secname_len-1] = '\0';
@@ -723,7 +724,11 @@ vacm_gen_com2sec(int commcount, const char *community, const char *addressname,
         snprintf(viewname, viewname_len-1, "viewComm%d", commcount);
         viewname[viewname_len-1] = '\0';
     }
-    snprintf(line, sizeof(line), "%s %s '%s'",
+    if ( context && *context )
+       snprintf(line, sizeof(line), "-Cn %s %s %s '%s'",
+             context, secname, addressname, community);
+    else
+       snprintf(line, sizeof(line), "%s %s '%s'",
              secname, addressname, community);
     line[ sizeof(line)-1 ] = 0;
     DEBUGMSGTL((publishtoken, "passing: %s %s\n", publishtoken, line));
@@ -943,18 +948,24 @@ vacm_create_simple(const char *token, char *confline,
     commcount++;
 
 #if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
+#ifdef NETSNMP_TRANSPORT_UDP_DOMAIN
     if (parsetype == VACM_CREATE_SIMPLE_COMIPV4 ||
         parsetype == VACM_CREATE_SIMPLE_COM) {
         vacm_gen_com2sec(commcount, community, addressname,
                          "com2sec", &netsnmp_udp_parse_security,
                          secname, sizeof(secname),
-                         view_ptr, sizeof(viewname), commversion);
+                         view_ptr, sizeof(viewname), commversion, context);
     }
-    
+#endif
+
 #ifdef NETSNMP_TRANSPORT_UNIX_DOMAIN
     if (parsetype == VACM_CREATE_SIMPLE_COMUNIX ||
         parsetype == VACM_CREATE_SIMPLE_COM) {
-        snprintf(line, sizeof(line), "%s %s '%s'",
+        if ( *context )
+           snprintf(line, sizeof(line), "-Cn %s %s %s '%s'",
+             context, secname, addressname, community);
+        else
+            snprintf(line, sizeof(line), "%s %s '%s'",
                  secname, addressname, community);
         line[ sizeof(line)-1 ] = 0;
         DEBUGMSGTL((token, "passing: %s %s\n", "com2secunix", line));
@@ -968,7 +979,7 @@ vacm_create_simple(const char *token, char *confline,
         vacm_gen_com2sec(commcount, community, addressname,
                          "com2sec6", &netsnmp_udp6_parse_security,
                          secname, sizeof(secname),
-                         view_ptr, sizeof(viewname), commversion);
+                         view_ptr, sizeof(viewname), commversion, context);
     }
 #endif
 #endif /* support for community based SNMP */
@@ -993,7 +1004,7 @@ vacm_create_simple(const char *token, char *confline,
              */
             snprintf(grpname, sizeof(grpname), "grp%.28s", secname);
             for (tmp=grpname; *tmp; tmp++)
-                if (!isalnum(*tmp))
+                if (!isalnum((unsigned char)(*tmp)))
                     *tmp = '_';
             snprintf(line, sizeof(line),
                      "%s %s \"%s\"", grpname, model, secname);
@@ -1004,7 +1015,7 @@ vacm_create_simple(const char *token, char *confline,
     } else {
         snprintf(grpname, sizeof(grpname), "grp%.28s", secname);
         for (tmp=grpname; *tmp; tmp++)
-            if (!isalnum(*tmp))
+            if (!isalnum((unsigned char)(*tmp)))
                 *tmp = '_';
     }
 
@@ -1111,10 +1122,12 @@ vacm_warn_if_not_configured(int majorID, int minorID, void *serverarg,
          */
         if ((MASTER_AGENT == agent_mode) && (strcmp(name, "snmptrapd") != 0)) {
             snmp_log(LOG_WARNING,
-                 "Warning: no access control information configured.\n  It's "
-                 "unlikely this agent can serve any useful purpose in this "
-                 "state.\n  Run \"snmpconf -g basic_setup\" to help you "
-                 "configure the %s.conf file for this agent.\n", name );
+                 "Warning: no access control information configured.\n"
+                 "  (Config search path: %s)\n"
+                 "  It's unlikely this agent can serve any useful purpose in this state.\n"
+                 "  Run \"snmpconf -g basic_setup\" to help you "
+                 "configure the %s.conf file for this agent.\n",
+                 get_configuration_directory(), name);
         }
 
         /*
@@ -1134,7 +1147,9 @@ vacm_warn_if_not_configured(int majorID, int minorID, void *serverarg,
                                     NETSNMP_DS_APP_NO_AUTHORIZATION)) {
             snmp_log(LOG_WARNING,
                  "Warning: no access control information configured.\n"
-                 "This receiver will *NOT* accept any incoming notifications.\n");
+                 "  (Config search path: %s)\n"
+                 "This receiver will *NOT* accept any incoming notifications.\n",
+                 get_configuration_directory());
         }
     }
     return SNMP_ERR_NOERROR;
@@ -1249,8 +1264,8 @@ vacm_check_view_contents(netsnmp_pdu *pdu, oid * name, size_t namelen,
     struct vacm_groupEntry *gp;
     struct vacm_viewEntry *vp;
     char            vacm_default_context[1] = "";
-    char           *contextName = vacm_default_context;
-    char           *sn = NULL;
+    const char     *contextName = vacm_default_context;
+    const char     *sn = NULL;
     char           *vn;
     const char     *pdu_community;
 
@@ -1271,7 +1286,7 @@ vacm_check_view_contents(netsnmp_pdu *pdu, oid * name, size_t namelen,
 #endif
 #endif
     {
-        pdu_community = pdu->community;
+        pdu_community = (const char *) pdu->community;
         if (!pdu_community)
             pdu_community = "";
         if (snmp_get_do_debugging()) {
@@ -1297,7 +1312,9 @@ vacm_check_view_contents(netsnmp_pdu *pdu, oid * name, size_t namelen,
          * community string to a security name for us.  
          */
 
-        if (pdu->tDomain == netsnmpUDPDomain
+        if (0) {
+#ifdef NETSNMP_TRANSPORT_UDP_DOMAIN
+        } else if (pdu->tDomain == netsnmpUDPDomain
 #ifdef NETSNMP_TRANSPORT_TCP_DOMAIN
             || pdu->tDomain == netsnmp_snmpTCPDomain
 #endif
@@ -1316,6 +1333,7 @@ vacm_check_view_contents(netsnmp_pdu *pdu, oid * name, size_t namelen,
             SNMP_FREE(pdu->contextName);
             pdu->contextName = strdup(contextName);
             pdu->contextNameLen = strlen(contextName);
+#endif
 #ifdef NETSNMP_TRANSPORT_UDPIPV6_DOMAIN
         } else if (pdu->tDomain == netsnmp_UDPIPv6Domain
 #ifdef NETSNMP_TRANSPORT_TCPIPV6_DOMAIN
