@@ -164,7 +164,7 @@ check_log_size(unsigned int clientreg, void *clientarg)
         data = (netsnmp_table_data_set_storage *) row->data;
         data = netsnmp_table_data_set_find_column(data, COLUMN_NLMLOGTIME);
 
-        if (uptime < ((long)(*(data->data.integer) + max_age * 100 * 60)))
+        if (uptime < ((u_long)(*(data->data.integer) + max_age * 100 * 60)))
             break;
         ++count;
     }
@@ -217,15 +217,6 @@ initialize_table_nlmLogVariableTable(const char * context)
                 "adding index nlmLogVariableIndex of type ASN_UNSIGNED to table nlmLogVariableTable\n"));
     netsnmp_table_dataset_add_index(table_set, ASN_UNSIGNED);
 
-    /*
-     * adding column nlmLogVariableIndex of type ASN_UNSIGNED and access
-     * of NoAccess 
-     */
-    DEBUGMSGTL(("initialize_table_nlmLogVariableTable",
-                "adding column nlmLogVariableIndex (#1) of type ASN_UNSIGNED to table nlmLogVariableTable\n"));
-    netsnmp_table_set_add_default_row(table_set,
-                                      COLUMN_NLMLOGVARIABLEINDEX,
-                                      ASN_UNSIGNED, 0, NULL, 0);
     /*
      * adding column nlmLogVariableID of type ASN_OBJECT_ID and access of
      * ReadOnly 
@@ -585,9 +576,11 @@ log_notification(netsnmp_pdu *pdu, netsnmp_transport *transport)
     u_long          vbcount = 0;
     u_long          tmpul;
     int             col;
+    netsnmp_pdu    *orig_pdu = pdu;
 
-    if (netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID,
-                               NETSNMP_DS_APP_DONT_LOG)) {
+    if (!nlmLogVarTable
+        || netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID,
+                                  NETSNMP_DS_APP_DONT_LOG)) {
         return;
     }
 
@@ -611,7 +604,7 @@ log_notification(netsnmp_pdu *pdu, netsnmp_transport *transport)
     gettimeofday(&now, NULL);
     tmpl = netsnmp_timeval_uptime(&now);
     netsnmp_set_row_column(row, COLUMN_NLMLOGTIME, ASN_TIMETICKS,
-                           (u_char *) & tmpl, sizeof(tmpl));
+                           &tmpl, sizeof(tmpl));
     time(&timetnow);
     logdate = date_n_time(&timetnow, &logdate_size);
     netsnmp_set_row_column(row, COLUMN_NLMLOGDATEANDTIME, ASN_OCTET_STR,
@@ -641,22 +634,23 @@ log_notification(netsnmp_pdu *pdu, netsnmp_transport *transport)
     }
     if (transport)
         netsnmp_set_row_column(row, COLUMN_NLMLOGENGINETDOMAIN,
-                                     ASN_OBJECT_ID,
-                                     (const u_char *) transport->domain,
-                                     sizeof(oid) * transport->domain_length);
+                               ASN_OBJECT_ID,
+                               transport->domain,
+                               sizeof(oid) * transport->domain_length);
     netsnmp_set_row_column(row, COLUMN_NLMLOGCONTEXTENGINEID,
                            ASN_OCTET_STR, pdu->contextEngineID,
                            pdu->contextEngineIDLen);
     netsnmp_set_row_column(row, COLUMN_NLMLOGCONTEXTNAME, ASN_OCTET_STR,
                            pdu->contextName, pdu->contextNameLen);
 
+    if (pdu->command == SNMP_MSG_TRAP)
+	pdu = convert_v1pdu_to_v2(orig_pdu);
     for (vptr = pdu->variables; vptr; vptr = vptr->next_variable) {
         if (snmp_oid_compare(snmptrapoid, snmptrapoid_len,
                              vptr->name, vptr->name_length) == 0) {
             netsnmp_set_row_column(row, COLUMN_NLMLOGNOTIFICATIONID,
                                    ASN_OBJECT_ID, vptr->val.string,
                                    vptr->val_len);
-
         } else {
             netsnmp_table_row *myrow;
             myrow = netsnmp_create_table_data_row();
@@ -676,7 +670,7 @@ log_notification(netsnmp_pdu *pdu, netsnmp_transport *transport)
              * OID 
              */
             netsnmp_set_row_column(myrow, COLUMN_NLMLOGVARIABLEID,
-                                   ASN_OBJECT_ID, (u_char *) vptr->name,
+                                   ASN_OBJECT_ID, vptr->name,
                                    vptr->name_length * sizeof(oid));
 
             /*
@@ -738,7 +732,7 @@ log_notification(netsnmp_pdu *pdu, netsnmp_transport *transport)
                 continue;
             }
             netsnmp_set_row_column(myrow, COLUMN_NLMLOGVARIABLEVALUETYPE,
-                                   ASN_INTEGER, (u_char *) & tmpul,
+                                   ASN_INTEGER, & tmpul,
                                    sizeof(tmpul));
             netsnmp_set_row_column(myrow, col, vptr->type,
                                    vptr->val.string, vptr->val_len);
@@ -748,6 +742,9 @@ log_notification(netsnmp_pdu *pdu, netsnmp_transport *transport)
         }
     }
 
+    if (pdu != orig_pdu)
+        snmp_free_pdu( pdu );
+
     /*
      * store the row 
      */
